==> 0.0.78.tar.gz: lib/src/widgets/data_factory/common/visual_data_processor.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/verttice_ui.dart';

import 'utils.dart';

class VisualData {
  final Widget widget;
  final String value;

  VisualData({
    required this.widget,
    required this.value,
  });
}

VisualData visualDataProcessor(
  FormItem item,
  Map<String, dynamic> row, {
  bool lookForPathToFieldOrItemBuilder = true,
  required BuildContext context,
}) {
  dynamic value = row[item.field.name];

  if (lookForPathToFieldOrItemBuilder && item.field.dataBuilder != null) {
    value = item.field.dataBuilder!(row);
  } else if (lookForPathToFieldOrItemBuilder &&
      item.field.pathToField != null &&
      item.field.pathToField!.isNotEmpty) {
    value = fieldFromPathToField(
      item.field.pathToField!,
      row,
    );
  }

  String returnValue = "";
  Widget? widget;
  if (item.field.showOnTable && item.field.name != "") {
    if (item.field is ColorSelectorFF) {
      SelectorColors colorSelectorValue = SelectorColors.grey;
      for (var selectorColor in SelectorColors.values) {
        if (selectorColor.name == value) {
          colorSelectorValue = selectorColor;
        }
      }
      late Color color;
      switch (colorSelectorValue) {
        case SelectorColors.green:
          color = Colors.green;
          break;
        case SelectorColors.red:
          color = Colors.red;
          break;

        case SelectorColors.orange:
          color = Colors.orange;
          break;

        case SelectorColors.grey:
          color = Colors.grey;
          break;

        case SelectorColors.yellow:
          color = Colors.yellow;
          break;
      }

      return VisualData(
        value: colorSelectorValue.name,
        widget: Container(
          color: color,
        ),
      );
    } else if (item.field is DateFF ||
        item.field is ReadonlyDateFF ||
        item.field is TimeFF ||
        item.field is ReadonlyTimeFF ||
        item.field is ReadonlyDateTimeFF ||
        item.field is TextFF ||
        item.field is TextBoxFF ||
        item.field is NumberFF ||
        item.field is ReadonlyNumberFF ||
        item.field is DateRangeFF ||
        item.field is ReadonlyDateRangeFF ||
        item.field is DateRangeFF ||
        item.field is DateTimeRangeFF ||
        item.field is ReadonlyDateTimeRangeFF) {
      String formattedValue = "";
      if (item.field is NumberFF || item.field is ReadonlyNumberFF) {
        formattedValue =
            DataClassUtils.dynamicToDouble(value)!.toStringAsFixed(2);
      } else if (item.field is DateFF || item.field is ReadonlyDateFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: true,
            time: false);
      } else if (item.field is TimeFF || item.field is ReadonlyTimeFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: false,
            time: true);
      } else if (item.field is ReadonlyDateTimeFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: true,
            time: true);
      } else if (item.field is DateRangeFF ||
          item.field is ReadonlyDateRangeFF) {
        formattedValue = value == null || value is! Map
            ? ""
            : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["startDate"], defultValue: 0)!, time: false) : ""} - ${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["endDate"], defultValue: 0)!, time: false) : ""}";
      } else if (item.field is DateTimeRangeFF ||
          item.field is ReadonlyDateTimeRangeFF) {
        formattedValue = value == null || value is! Map
            ? ""
            : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["startDate"], defultValue: 0)!) : ""} - \n${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["endDate"], defultValue: 0)!) : ""}";
      } else {
        formattedValue = DataClassUtils.dynamicToString(value);
      }

      returnValue = formattedValue;
    } else if (item.field is PhoneNumberFF) {
      final address = visualPhoneNumber(value, context: context);
      returnValue = address.value;
      widget = address.widget;
    } else if (item.field is PhoneNumberListFF) {
      final address = visualPhoneNumberList(value, context: context);
      returnValue = address.value;
      widget = address.widget;
    } else if (item.field is AddressOnMapFF) {
      final address = visualAddressOnMap(value, context: context);
      returnValue = address.value;
      widget = address.widget;
    } else if (item.field is EmailListFF || item.field is TextListFF) {
      returnValue = value is List ? value.join("\n") : "";
    } else {
      returnValue = DataClassUtils.dynamicToString(value);
    }
  }

  return VisualData(
    value: returnValue,
    widget: widget ??
        InkWell(
          onTap: () {
            Clipboard.setData(
              ClipboardData(
                text: returnValue,
              ),
            );
            ThemedToast.info(
              context: context,
              message: "Copied to clipboard",
            );
          },
          child: Text(
            returnValue,
            style: const TextStyle(
              fontSize: 12,
            ),
          ),
        ),
  );
}

VisualData visualAddressOnMap(
  dynamic value, {
  required BuildContext context,
}) {
  final address = AddressOnMap.fromMap(
    DataClassUtils.dynamicToMapStringDynamic(value),
  );
  final returnValue = address.address != null && address.address != ''
      ? '📍 ${address.address}'
      : "";

  return VisualData(
      value: returnValue,
      widget: Row(
        children: [
          if (address.websiteUri != null)
            Tooltip(
              message: 'Open address on google maps',
              child: IconButton(
                onPressed: () async {
                  Clipboard.setData(
                    ClipboardData(
                      text: returnValue,
                    ),
                  );
                  ThemedToast.info(
                    context: context,
                    message: "Address copied to clipboard",
                  );

                  if (address.websiteUri != null &&
                      await canLaunch(address.websiteUri ?? '')) {
                    launch(address.websiteUri!);
                  }
                },
                icon: const Icon(
                  Icons.location_on_outlined,
                ),
              ),
            ),
          if (address.websiteUri != null) sizedBoxW4,
          InkWell(
            onTap: () async {
              Clipboard.setData(
                ClipboardData(
                  text: returnValue,
                ),
              );
              ThemedToast.info(
                context: context,
                message: "Address copied to clipboard",
              );
            },
            child: Text(
              returnValue,
              style: const TextStyle(
                fontSize: 12,
              ),
            ),
          )
        ],
      ));
}

VisualData visualPhoneNumber(
  dynamic value, {
  required BuildContext context,
}) {
  final phone = PhoneNumberModel.fromMap(
    DataClassUtils.dynamicToMapStringDynamic(value),
  );
  final returnValue =
      "${phone.flag}${phone.flag != null && phone.flag!.isNotEmpty ? " " : ""}${phone.completeNumberWith0 ?? ''}";

  return VisualData(
    value: returnValue,
    widget: InkWell(
      onTap: () {
        Clipboard.setData(
          ClipboardData(
            text: phone.completeNumber,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Phone number copied to clipboard",
        );
      },
      child: Text(
        returnValue,
        style: const TextStyle(
          fontSize: 12,
        ),
      ),
    ),
  );
}

VisualData visualPhoneNumberList(
  dynamic value, {
  required BuildContext context,
}) {
  final returnValue = value is List
      ? value.map((e) {
          final phone = PhoneNumberModel.fromMap(
            DataClassUtils.dynamicToMapStringDynamic(e),
          );
          return "${phone.flag}${phone.flag != null && phone.flag!.isNotEmpty ? " " : ""}${phone.completeNumberWith0 ?? ''}${(phone.flag != null && phone.flag!.isNotEmpty) || (phone.completeNumberWith0 != null && phone.completeNumberWith0!.isNotEmpty) ? "\n" : ""}";
        }).join("")
      : "";

  final copyValue = value is List
      ? value.map((e) {
          final phone = PhoneNumberModel.fromMap(
            DataClassUtils.dynamicToMapStringDynamic(e),
          );
          return "${phone.completeNumber ?? ''}${(phone.completeNumber != null && phone.completeNumber!.isNotEmpty) ? "\n" : ""}";
        }).join("")
      : "";
  return VisualData(
    value: returnValue,
    widget: InkWell(
      onTap: () {
        Clipboard.setData(
          ClipboardData(
            text: copyValue,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Phone numbers copied to clipboard",
        );
      },
      child: SelectableText(
        returnValue,
        style: const TextStyle(
          fontSize: 12,
        ),
      ),
    ),
  );
}

==> 0.0.80.tar.gz: lib/src/widgets/data_factory/common/visual_data_processor.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/verttice_ui.dart';

class VisualData {
  final Widget widget;
  final String value;

  VisualData({
    required this.widget,
    required this.value,
  });
}

VisualData visualDataProcessor(
  FormItem item,
  Map<String, dynamic> row, {
  bool lookForPathToFieldOrItemBuilder = true,
  required BuildContext context,
}) {
  dynamic value = row[item.field.name];

  if (lookForPathToFieldOrItemBuilder && item.field.dataBuilder != null) {
    value = item.field.dataBuilder!(row);
  } else if (lookForPathToFieldOrItemBuilder &&
      item.field.pathToField != null &&
      item.field.pathToField!.isNotEmpty) {
    value = fieldFromPathToField(
      item.field.pathToField!,
      row,
    );
  }

  String returnValue = "";
  Widget? widget;
  if (item.field.showOnTable && item.field.name != "") {
    if (item.field is ColorSelectorFF) {
      SelectorColors colorSelectorValue = SelectorColors.grey;
      for (var selectorColor in SelectorColors.values) {
        if (selectorColor.name == value) {
          colorSelectorValue = selectorColor;
        }
      }
      late Color color;
      switch (colorSelectorValue) {
        case SelectorColors.green:
          color = Colors.green;
          break;
        case SelectorColors.red:
          color = Colors.red;
          break;

        case SelectorColors.orange:
          color = Colors.orange;
          break;

        case SelectorColors.grey:
          color = Colors.grey;
          break;

        case SelectorColors.yellow:
          color = Colors.yellow;
          break;
      }

      return VisualData(
        value: colorSelectorValue.name,
        widget: Container(
          color: color,
        ),
      );
    } else if (item.field is DateFF ||
        item.field is ReadonlyDateFF ||
        item.field is TimeFF ||
        item.field is ReadonlyTimeFF ||
        item.field is ReadonlyDateTimeFF ||
        item.field is TextFF ||
        item.field is TextBoxFF ||
        item.field is NumberFF ||
        item.field is ReadonlyNumberFF ||
        item.field is DateRangeFF ||
        item.field is ReadonlyDateRangeFF ||
        item.field is DateRangeFF ||
        item.field is DateTimeRangeFF ||
        item.field is ReadonlyDateTimeRangeFF) {
      String formattedValue = "";
      if (item.field is NumberFF || item.field is ReadonlyNumberFF) {
        formattedValue =
            DataClassUtils.dynamicToDouble(value)!.toStringAsFixed(2);
      } else if (item.field is DateFF || item.field is ReadonlyDateFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: true,
            time: false);
      } else if (item.field is TimeFF || item.field is ReadonlyTimeFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: false,
            time: true);
      } else if (item.field is ReadonlyDateTimeFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: true,
            time: true);
      } else if (item.field is DateRangeFF ||
          item.field is ReadonlyDateRangeFF) {
        formattedValue = value == null || value is! Map
            ? ""
            : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["startDate"], defultValue: 0)!, time: false) : ""} - ${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["endDate"], defultValue: 0)!, time: false) : ""}";
      } else if (item.field is DateTimeRangeFF ||
          item.field is ReadonlyDateTimeRangeFF) {
        formattedValue = value == null || value is! Map
            ? ""
            : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["startDate"], defultValue: 0)!) : ""} - \n${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["endDate"], defultValue: 0)!) : ""}";
      } else {
        formattedValue = DataClassUtils.dynamicToString(value);
      }

      returnValue = formattedValue;
    } else if (item.field is PhoneNumberFF) {
      final phone = visualPhoneNumber(value, context: context);
      returnValue = phone.value;
      widget = phone.widget;
    } else if (item.field is PhoneNumberListFF) {
      final phones = visualPhoneNumberList(value, context: context);
      returnValue = phones.value;
      widget = phones.widget;
    } else if (item.field is AddressOnMapFF) {
      final address = visualAddressOnMap(value, context: context);
      returnValue = address.value;
      widget = address.widget;
    } else if (item.field is EmailListFF || item.field is TextListFF) {
      returnValue = value is List ? value.join("\n") : "";
    } else {
      returnValue = DataClassUtils.dynamicToString(value);
    }
  }

  return VisualData(
    value: returnValue,
    widget: widget ??
        InkWell(
          onTap: () {
            Clipboard.setData(
              ClipboardData(
                text: returnValue,
              ),
            );
            ThemedToast.info(
              context: context,
              message: "Copied to clipboard",
            );
          },
          child: Text(
            returnValue,
            style: const TextStyle(
              fontSize: 12,
            ),
          ),
        ),
  );
}

VisualData visualAddressOnMap(
  dynamic value, {
  required BuildContext context,
}) {
  final address = AddressOnMap.fromMap(
    DataClassUtils.dynamicToMapStringDynamic(value),
  );
  final copyValue = address.fullAddress != null && address.fullAddress != ''
      ? '📍 ${address.fullAddress}'
      : "";
  final returnValue = address.fullAddress ?? "";

  return VisualData(
      value: returnValue,
      widget: Tooltip(
        message: copyValue,
        child: LayoutBuilder(builder: (context, constraints) {
          return SizedBox(
            width: constraints.maxWidth,
            child: Row(
              children: [
                if (address.websiteUri != null && address.websiteUri != '')
                  SizedBox(
                    width: 10,
                    child: Center(
                      child: Tooltip(
                        message: 'Open address on google maps',
                        child: InkWell(
                          onTap: () async {
                            if (address.websiteUri != null &&
                                (await canLaunch(address.websiteUri ?? ''))) {
                              //https://www.google.com/maps/search/?api=1&query=<lat>,<lng>
                              await launch(address.websiteUri!);
                            }
                          },
                          child: const Text(
                            '📍 ',
                          ),
                        ),
                      ),
                    ),
                  ),
                InkWell(
                  onTap: () async {
                    Clipboard.setData(
                      ClipboardData(
                        text: copyValue,
                      ),
                    );
                    ThemedToast.info(
                      context: context,
                      message: "Address copied to clipboard",
                    );
                  },
                  child: SizedBox(
                    width: (address.websiteUri != null)
                        ? constraints.maxWidth - 12
                        : constraints.maxWidth,
                    child: Text(
                      returnValue,
                      overflow: TextOverflow.fade,
                      style: const TextStyle(
                        fontSize: 11.5,
                      ),
                    ),
                  ),
                )
              ],
            ),
          );
        }),
      ));
}

VisualData visualPhoneNumber(
  dynamic value, {
  required BuildContext context,
}) {
  final phone = PhoneNumberModel.fromMap(
    DataClassUtils.dynamicToMapStringDynamic(value),
  );
  final returnValue =
      "${phone.flag}${phone.flag != null && phone.flag!.isNotEmpty ? " " : ""}${phone.completeNumberWith0 ?? ''}";

  return VisualData(
    value: returnValue,
    widget: InkWell(
      onTap: () {
        Clipboard.setData(
          ClipboardData(
            text: phone.completeNumber,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Phone number copied to clipboard",
        );
      },
      child: Text(
        returnValue,
        style: const TextStyle(
          fontSize: 12,
        ),
      ),
    ),
  );
}

VisualData visualPhoneNumberList(
  dynamic value, {
  required BuildContext context,
}) {
  final returnValue = value is List
      ? value.map((e) {
          final phone = PhoneNumberModel.fromMap(
            DataClassUtils.dynamicToMapStringDynamic(e),
          );
          return "${phone.flag}${phone.flag != null && phone.flag!.isNotEmpty ? " " : ""}${phone.completeNumberWith0 ?? ''}${(phone.flag != null && phone.flag!.isNotEmpty) || (phone.completeNumberWith0 != null && phone.completeNumberWith0!.isNotEmpty) ? "\n" : ""}";
        }).join("")
      : "";

  final copyValue = value is List
      ? value.map((e) {
          final phone = PhoneNumberModel.fromMap(
            DataClassUtils.dynamicToMapStringDynamic(e),
          );
          return "${phone.completeNumber ?? ''}${(phone.completeNumber != null && phone.completeNumber!.isNotEmpty) ? "\n" : ""}";
        }).join("")
      : "";
  return VisualData(
    value: returnValue,
    widget: InkWell(
      onTap: () {
        Clipboard.setData(
          ClipboardData(
            text: copyValue,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Phone numbers copied to clipboard",
        );
      },
      child: SelectableText(
        returnValue,
        style: const TextStyle(
          fontSize: 12,
        ),
      ),
    ),
  );
}
